@@ -1,6 +1,6 @@
 import collections.abc
 import sys
-from typing import Any, Callable, Dict, List, Set, Union, cast
+from typing import Any, Callable, Dict, List, Set, Tuple, Union, cast
 
 from .types import Listx, Tuplex
 from .typing_compat import (
@@ -12,7 +12,6 @@ from .typing_compat import (
     get_args,
     get_origin,
     get_type_hints,
-    is_generic,
     is_literal,
     is_newtype,
     is_typeddict,
@@ -47,21 +46,42 @@ def _isinstancex(obj: Any, tp: TypeLike) -> bool:
     # convert
     # - a plain dictionary to Dict or TypedDict
     # - a plain list to Listx[...]
+    # - a plain tuple to Tuplex[...]
     if origin is None:
+        # tp is of form `{'a': TypeLike, ...}`, `{...: TypeLike}`
         if isinstance(tp, dict):
             tp = {(TYPED_DICT_EXTRA_KEY if k is ... else k): v for k, v in tp.items()}
-            if len(tp) == 1 and not isinstancex(list(tp.keys())[0], str):
-                # tp is of form `{TypeLike: TypeLike}`
-                return _isinstancex(obj, Dict[list(tp.items())[0]])  # type: ignore
-            else:
-                # tp is of form `{'a': TypeLike, ...}`, `{...: TypeLike}`
-                return _isinstancex(obj, TypedDict("_TypedDict", tp))  # type: ignore
+            return isinstancex(obj, TypedDict("_TypedDict", tp))  # type: ignore
         elif isinstance(tp, list):
-            return _isinstancex(obj, Listx[tuple(tp)])
+            return isinstancex(obj, Listx[tuple(tp)])
+        elif isinstance(tp, tuple):
+            return isinstancex(obj, Tuplex[tuple(tp)])
 
     # e.g. Union[str, int] (or str|int in 3.10)
     if origin in UNION_TYPES:
-        return isinstancex(obj, get_args(tp))
+        return any(isinstancex(obj, arg) for arg in get_args(tp))
+
+    # e.g. Callable[[int], str]
+    elif origin is collections.abc.Callable:
+        if not callable(obj):
+            return False
+
+        expected_args_types, expected_return_type = get_args(tp) or (..., Any)
+
+        if expected_args_types is ... and expected_return_type is Any:
+            return True
+
+        args_types, return_type = _get_function_type_hints(obj)
+
+        if not issubclassx(return_type, expected_return_type):
+            return False
+
+        if expected_args_types is ...:
+            return True
+
+        return len(args_types) == len(expected_args_types) and all(
+            issubclassx(a_tp, e_tp) for (a_tp, e_tp) in zip(args_types, expected_args_types)
+        )
 
     # e.g. Dict[str, int]
     elif origin is dict:
@@ -75,7 +95,10 @@ def _isinstancex(obj: Any, tp: TypeLike) -> bool:
         if tp is List:
             tp = List[Any]
 
-        return isinstancex(obj, list) and _is_valid_sequence(obj, tp, is_list=True)
+        name = getattr(tp, "_name", None) or getattr(tp, "__name__", None)
+
+        # We consider Listx[int] to check if a list as ONLY ONE item
+        return isinstancex(obj, list) and _is_valid_sequence(obj, tp, is_list=name != "Listx")
 
     # e.g. Set[str]
     elif origin is set:
@@ -83,7 +106,7 @@ def _isinstancex(obj: Any, tp: TypeLike) -> bool:
         if tp is Set:
             tp = Set[Any]
 
-        return isinstancex(obj, set) and all(isinstancex(x, get_args(tp)) for x in obj)
+        return isinstancex(obj, set) and all(isinstancex(x, Union[get_args(tp)]) for x in obj)
 
     # e.g. Tuple[int, ...] or Tuplex[int, str, ...]
     elif origin is tuple:
@@ -91,7 +114,7 @@ def _isinstancex(obj: Any, tp: TypeLike) -> bool:
 
     # e.g. Type[int]
     elif origin is type:
-        return issubclassx(obj, get_args(tp))
+        return issubclassx(obj, Union[get_args(tp)])
 
     # e.g. TypedDict('Movie', {'name': str, 'year': int})
     elif is_typeddict(tp):
@@ -123,35 +146,32 @@ def _isinstancex(obj: Any, tp: TypeLike) -> bool:
 
 
 def _issubclassx(obj: Any, tp: TypeLike) -> bool:
-    if obj is tp:
+    if tp is Any:
         return True
 
-    if is_generic(obj) and is_generic(tp):
-        obj_mother_class = get_origin(obj.__orig_bases__[0])
-        obj_mother_args = get_args(obj.__orig_bases__[0])
+    if obj in NONE_TYPES and tp in NONE_TYPES:
+        return True
 
-        return issubclassx(obj_mother_class, get_origin(tp)) and all(
-            issubclassx(arg, get_args(tp)) for arg in obj_mother_args
-        )
+    origin = get_origin(tp)
+
+    if origin in UNION_TYPES:
+        return any(issubclassx(obj, arg) for arg in get_args(tp))
 
     return issubclass(obj, tp)
 
 
-def _safe_multi(f: Callable[[Any, TypeLike], bool]) -> Callable[[Any, OneOrManyTypes], bool]:
-    def safe_f_multi(obj: Any, tp: OneOrManyTypes) -> bool:
+def _safe(f: Callable[[Any, TypeLike], bool]) -> Callable[[Any, OneOrManyTypes], bool]:
+    def safe_f(obj: Any, tp: OneOrManyTypes) -> bool:
         try:
-            if isinstance(tp, tuple):
-                return any(f(obj, sub_tp) for sub_tp in tp)
-            else:
-                return f(obj, tp)
+            return f(obj, tp)
         except (AttributeError, TypeError):
             return False
 
-    return safe_f_multi
+    return safe_f
 
 
-isinstancex = _safe_multi(_isinstancex)
-issubclassx = _safe_multi(_issubclassx)
+isinstancex = _safe(_isinstancex)
+issubclassx = _safe(_issubclassx)
 
 
 #######################################
@@ -227,3 +247,33 @@ def _is_valid_typeddict(obj: Any, tp: TypedDict) -> bool:
             return False
 
         return all(isinstancex(v, resolved_annotations[k]) for k, v in obj.items())
+
+
+def _get_function_type_hints(obj: Callable[..., Any]) -> Tuple[List[TypeLike], TypeLike]:
+    """Return a tuple <list of types of arguments>, <return type>"""
+    import inspect
+    import warnings
+
+    sig = inspect.signature(obj)
+
+    args_types: List[TypeLike] = []
+
+    for name, parameter in sig.parameters.items():
+        tp = parameter.annotation
+        if tp is sig.empty:
+            warnings.warn(
+                f"No type hint specified for arg {name!r} of {obj.__name__!r}...fallback on `Any`",
+                UserWarning,
+            )
+            tp = Any
+        args_types.append(tp)
+
+    return_type = sig.return_annotation
+    if return_type is sig.empty:
+        warnings.warn(
+            f"No return type hint specified for {obj.__name__!r}...fallback on `Any`",
+            UserWarning,
+        )
+        return_type = Any
+
+    return args_types, return_type
